import json
import os

import arrow
import time

from WebMonitoring.WebsiteMonitorHelpers.request_helpers import parse_request
from WebMonitoring.clients.SpeedprofileClient import SpeedprofileClient
from WebMonitoring.configs.constants import sample_url, sample_name, DockerResults
from WebMonitoring.configs.settings import SHOW_VERBOSE_MESSAGE, json_path
"""Run advanced monitoring on websites including total response time."""


class WebsiteMonitor:
    def __init__(self, url, name):
        self.url = url
        self.name = name

        # Get current timestamp
        self.timestamp = time.localtime()

        self.startTimes = []
        self.endTimes = []
        self.start = None
        self.end = None
        self.total_time = None
        self.total_time_seconds = 0.0

        # List of RequestEntry structures, containing metrics
        self.request_entry = []

        self.docker = None

    def calculate_total_time(self):
        """Calculate total requests time as difference between last recorded response and
        first request recorded time"""

        if not self.startTimes or not self.request_entry:
            # Return -1 if there are no start times recorded or requests entries
            self.total_time_seconds = -1
            return

        self.start = min(self.startTimes)
        self.end = max(self.endTimes)
        self.total_time = self.end - self.start

        self.total_time_seconds = (self.total_time.microseconds / 1000000.0 +
                                   self.total_time.seconds +
                                   self.total_time.days * 86400)

    def add_times(self, entry):
        """Calculate a total request time as startedDateTime + time"""
        requestStartTime = arrow.get(entry["startedDateTime"])
        requestEndTime = arrow.get(
            entry["startedDateTime"]).shift(microseconds=1000 * entry["time"])
        self.startTimes.append(requestStartTime)
        self.endTimes.append(requestEndTime)

    def process_json(self):
        # Check existence of output file
        if not os.path.isfile(json_path):
            return

        # If file has been generated, proceed to parsing
        with open(json_path, encoding="utf-8") as json_file:
            data = json.load(json_file)
            for entry in data["log"]["entries"]:
                self.add_times(entry)
                request_entry = parse_request(entry)
                self.request_entry.append(request_entry)

        # Remove file after processing
        os.remove(json_path)

    def get_metrics(self):
        """Return a list of RequestEntry structures containing metrics"""
        return self.request_entry

    def run(self):
        """Generate timestamp at beginning of script"""
        self.timestamp = time.localtime()

        # Run the docker client containing the Chrome browser and export the HAR file
        self.docker = SpeedprofileClient(self.url)
        self.docker.run()

        if SHOW_VERBOSE_MESSAGE:
            print("Docker run status: " + self.docker.status)

        self.process_json()
        self.calculate_total_time()
        self.verbose()

        return self.docker.status

    def verbose(self):
        if SHOW_VERBOSE_MESSAGE:
            print("Start time: %s" % self.start)
            print("End time: %s" % self.end)
            print("Total time: %.3f" % self.total_time_seconds)

    def get_timestamp(self):
        """Return the timestamp corresponding to the execution time of the command"""
        return self.timestamp

    def get_total_time_seconds(self):
        """Return the total time for all requests processing"""
        return self.total_time_seconds


# Driver for WebsiteMonitor class
if __name__ == "__main__":
    monitor = WebsiteMonitor(sample_url, sample_name)
    monitor_result = monitor.run()

    # Check docker exit code
    if monitor_result == DockerResults.SUCCESS:
        print(monitor.get_metrics())
        print(monitor.get_timestamp())
        print(monitor.get_total_time_seconds())



import mysql.connector
import time

from WebMonitoring.configs.constants import DockerResults
from WebMonitoring.configs.settings import (
    SAMPLE_TIME,
    SHOW_VERBOSE_MESSAGE,
    MYSQL_DATABASE_USER,
    MYSQL_DATABASE_PASSWORD,
    MYSQL_DATABASE_DB,
    MYSQL_DATABASE_HOST,
)
from WebMonitoring.monitors.WebsiteMonitor import WebsiteMonitor


class WebsiteManager:
    def __init__(self, sample_time=3600):
        self.sample_time = sample_time
        self.websites = []
        self.cnx = None

    def run(self):
        # Show Websites to be monitored
        if SHOW_VERBOSE_MESSAGE:
            print("RUN: ", self.websites)

        for (website_url, website_name) in self.websites:

            monitor = WebsiteMonitor(website_url, website_name)
            monitor_result = monitor.run()

            # Check docker exit code
            if monitor_result == DockerResults.SUCCESS:
                if SHOW_VERBOSE_MESSAGE:
                    print(monitor.get_metrics())
                    print(monitor.get_timestamp())
                    print(monitor.get_total_time_seconds())

                cursor = self.cnx.cursor(buffered=True)
                query = (
                    "SELECT Websiteid, WebsiteName from WEBSITES where WebsiteUrl = '%s'"
                    % website_url)
                cursor.execute(query)

                # Get Websiteid
                for (website_id, _) in cursor:
                    query = (
                        "INSERT INTO WEBSITES_METRICS(Websiteid, TotalTime) "
                        "VALUES (%d, %lf)") % (
                            website_id, monitor.get_total_time_seconds())
                    cursor_insert = self.cnx.cursor(buffered=True)
                    # Insert new Website Metric Entry in DB
                    cursor_insert.execute(query)
                    self.cnx.commit()

                    cursor_metric_id = self.cnx.cursor(buffered=True)
                    query = (
                        "select max(Metricid) from WEBSITES_METRICS where Websiteid = %d"
                        % website_id)
                    cursor_metric_id.execute(query)

                    # Get Metricid
                    for (metric_id, ) in cursor_metric_id:
                        for request in monitor.request_entry:
                            query = (
                                "INSERT INTO REQUESTS(Metricid, serverIPAddress, pageRef, startedDateTime, time, responseStatus, headersSize, bodySize) "
                                "VALUES (%d, '%s', '%s', '%s', %d, %d, %d, %d )"
                            ) % (
                                metric_id,
                                request.serverIPAddress,
                                request.pageRef,
                                request.startedDateTime,
                                request.time,
                                request.responseStatus,
                                request.headersSize,
                                request.bodySize,
                            )
                            cursor_insert_request = self.cnx.cursor(
                                buffered=True)
                            # Insert new timing in DB
                            cursor_insert_request.execute(query)
                            self.cnx.commit()

                            cursor_request_id = self.cnx.cursor(buffered=True)
                            query = (
                                "select max(Requestid) from REQUESTS where Metricid = %d "
                                % metric_id)
                            cursor_request_id.execute(query)

                            # Get Requestid
                            for (request_id, ) in cursor_request_id:
                                timing = request.timing
                                cursor_insert_timing = self.cnx.cursor(
                                    buffered=True)
                                query = (
                                    "INSERT INTO TIMINGS(Requestid, Receive, Send, SSLTime, Connect, DNS, Blocked, Wait) VALUES (%d, %d, %d, %d, %d, %d, %d, %d)"
                                    % (
                                        request_id,
                                        request.timing.receive,
                                        timing.send,
                                        timing.ssl,
                                        timing.connect,
                                        timing.dns,
                                        timing.blocked,
                                        timing.wait,
                                    ))
                                # Insert new timing in DB
                                cursor_insert_timing.execute(query)
                                self.cnx.commit()

    def start(self):
        # Connect to DB
        self.cnx = mysql.connector.connect(
            user=MYSQL_DATABASE_USER,
            password=MYSQL_DATABASE_PASSWORD,
            host=MYSQL_DATABASE_HOST,
            database=MYSQL_DATABASE_DB,
        )

        while True:
            self.websites = []

            # Check for any update in the list of websites
            cursor = self.cnx.cursor(buffered=True)
            query = "SELECT WebsiteUrl, WebsiteName from WEBSITES"
            cursor.execute(query)

            for (website_url, website_name) in cursor:
                self.websites.append((website_url, website_name))

            self.run()
            time.sleep(self.sample_time)


if __name__ == "__main__":
    WebsiteManager(SAMPLE_TIME).start()



from WebMonitoring.configs.settings import SHOW_VERBOSE_MESSAGE


def parse_request(entry):
    serverIPAddress = get_server_ip(entry)
    pageRef = get_page_ref(entry)
    startedDateTime = get_started_date_time(entry)
    time = get_time(entry)
    responseStatus = get_response_status(entry)
    headersSize = get_headers_size(entry)
    bodySize = get_body_size(entry)

    timing = get_timing(entry)
    request_entry = RequestEntry(
        serverIPAddress,
        pageRef,
        startedDateTime,
        time,
        responseStatus,
        headersSize,
        bodySize,
        timing,
    )
    request_entry.verbose()
    return request_entry


def get_timing(entry):
    return entry.get("timings", {})


def get_server_ip(entry):
    return entry.get("serverIPAddress", "")


def get_page_ref(entry):
    return entry.get("pageref", "")


def get_started_date_time(entry):
    return entry.get("startedDateTime", "")


def get_time(entry):
    return entry.get("time", "")


def get_response_status(entry):
    return entry.get("response", {}).get("status", "")


def get_headers_size(entry):
    return entry.get("response", {}).get("headersSize", "")


def get_body_size(entry):
    return entry.get("response", {}).get("bodySize", "")


class RequestEntry:
    def __init__(
        self,
        serverIPAddress,
        pageRef,
        startedDateTime,
        time,
        responseStatus,
        headersSize,
        bodySize,
        timing,
    ):
        self.serverIPAddress = serverIPAddress
        self.pageRef = pageRef
        self.startedDateTime = startedDateTime
        self.time = time
        self.responseStatus = responseStatus
        self.headersSize = headersSize
        self.bodySize = bodySize
        self.timing = TimingEntry(timing)

    def get(self):
        return self

    def verbose(self):
        if SHOW_VERBOSE_MESSAGE:
            print("serverIPAddress: %s" % self.serverIPAddress)
            print("pageRef: %s" % self.pageRef)
            print("startedDateTime: %s" % self.startedDateTime)
            print("time: %d" % self.time)
            print("responseStatus: %d" % self.responseStatus)
            print("headersSize: %d" % self.headersSize)
            print("bodySize: %d" % self.bodySize)
            self.timing.verbose()


class TimingEntry:
    def __init__(self, timing):
        self.timing = self.parse_timing(timing)

    def parse_timing(self, timing):
        self.receive = timing.get("receive", "")
        self.send = timing.get("send", "")
        self.ssl = timing.get("ssl", "")
        self.connect = timing.get("connect", "")
        self.dns = timing.get("dns", "")
        self.blocked = timing.get("blocked", "")
        self.wait = timing.get("wait", "")

    def get(self):
        return self

    def verbose(self):
        if SHOW_VERBOSE_MESSAGE:
            print("receive: %d" % self.receive)
            print("send: %d" % self.send)
            print("ssl: %d" % self.ssl)
            print("connect: %d" % self.connect)
            print("dns: %d" % self.dns)
            print("blocked: %d" % self.blocked)
            print("wait: %d" % self.wait)



from flask import Flask
from flask import request
from flask_cors import CORS

app = Flask(__name__)
CORS(app)


@app.route("/")
def index():
    return "Hello, world!"


@app.route("/submit", methods=["POST"])
def add_user():
    _json = request.json
    try:
        program = _json["program"]
        name = _json["name"]
        data = _json["data"]
        if not program or not name or not data:
            return "Empty program/name/data"
        return "Request acknowledged"

    except Exception as e:
        print(e)
    finally:
        return "Request invalid! Must include program, name, data"


if __name__ == "__main__":
    app.run(host="0.0.0.0", debug=True)



import matplotlib.pyplot as plt
import sys
from sklearn.metrics import mean_squared_error, r2_score

from rafComputing.ML.CustomSettings.settings import (
    MAX_ITER,
    ALPHA,
    MAX_ITER_LinearRegressionTraining,
    ALPHA_LinearRegressionTraining,
    AUTO_DRIVER_DEFAULT_OUTPUT_NAME,
)
from rafComputing.ML.RegressionEngine.LinearRegressionGD import LinearRegressionGD
from rafComputing.ML.features.feature_types import (
    NO_FEATURE_TYPE,
    POWER_FEATURE_TYPE,
)
from rafComputing.ML.helpers.load_data import matrix_to_train_test

LINE_COLORS = ["blue", "orange", "green"]

# Light
TRAIN_POINT_COLORS = ["#ADD8E6", "#FFCF9E", "#90ee90"]
# Dark
TEST_POINT_COLORS = ["#00008B", "#b36200", "#006400"]


def LinearRegressionTrainingMoreGraphs(
    path,
    alpha=ALPHA_LinearRegressionTraining,
    n_iterations=MAX_ITER_LinearRegressionTraining,
    internal_counter=0,
    feature_type=NO_FEATURE_TYPE,
    feature_val=1.0,
):
    (x, orig_x), y, (X_train, orig_x_train), (
        X_test,
        orig_x_test,
    ), y_train, y_test = matrix_to_train_test(path=path,
                                              feature_type=feature_type,
                                              feature_val=feature_val)

    # Model initialization
    regression_model = LinearRegressionGD(alpha, n_iterations)
    regression_model.fit(X_train, y_train)
    print(regression_model.w_.flatten())

    # Predict
    y_predicted = regression_model.predict(x)
    y_predicted_train = regression_model.predict(X_train)
    y_predicted_test = regression_model.predict(X_test)

    # Model evaluation training data
    rmse = mean_squared_error(y_train, y_predicted_train)
    r2 = r2_score(y_train, y_predicted_train)
    print("[Training Set] Root mean squared error: ", rmse)
    print("[Training Set] R2 score: ", r2)

    # Model evaluation test data
    rmse = mean_squared_error(y_test, y_predicted_test)
    r2 = r2_score(y_test, y_predicted_test)
    print("[Test Set] Root mean squared error: ", rmse)
    print("[Test Set] R2 score: ", r2)

    orig_y_train = y_train.flatten()
    orig_y_test = y_test.flatten()

    # Data points
    plt.scatter(orig_x_train,
                orig_y_train,
                s=8,
                color=TRAIN_POINT_COLORS[internal_counter])
    plt.scatter(orig_x_test,
                orig_y_test,
                s=12,
                color=TEST_POINT_COLORS[internal_counter])
    plt.xlabel("Input size")
    plt.ylabel("Time (seconds)")

    # Plotting predicted values
    orig_x = orig_x.flatten()

    # Predicted values
    y_predicted = y_predicted.flatten()
    plt.plot(orig_x, y_predicted, color=LINE_COLORS[internal_counter])


# Ex. usage: python3 rafComputing/ML/snippets/MLdriver_more_graphs.py rComplexity/samples/matrix_multiplication/results/fsri5/result_n3_0_20200309165609 rComplexity/samples/matrix_multiplication/results/fsri5/result_n3_2_20200309165609 rComplexity/samples/matrix_multiplication/results/fsri5/result_n3_3_20200309165609
if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print(
            "Usage v1: python3 MLdriver_more_graphs.py <file_name1> <file_name2> <file_name3> ... "
        )
        print(
            "Usage (e.g.): python3 rafComputing/ML/snippets/MLdriver_more_graphs.py rComplexity/samples/matrix_multiplication/results/fsri5/result_n3_0_20200309165609 rComplexity/samples/matrix_multiplication/results/fsri5/result_n3_2_20200309165609 rComplexity/samples/matrix_multiplication/results/fsri5/result_n3_3_20200309165609             "
        )
        sys.exit(-1)

    counter = 0
    for path in sys.argv:
        if path == sys.argv[0]:
            continue

        LinearRegressionTrainingMoreGraphs(
            path=path,
            alpha=ALPHA,
            n_iterations=MAX_ITER,
            internal_counter=counter,
            feature_type=POWER_FEATURE_TYPE,
            feature_val=3.0,
        )

        counter = counter + 1

    plt.legend([
        "Regression line: Naive Matrix Multiplication",
        "Regression line: Cache-friendly loop ordering",
        "Regression line: Blocked Matrix Multiplication",
        "Train data: Naive Matrix Multiplication",
        "Test data:  Naive Matrix Multiplication",
        "Train data: Cache-friendly loop ordering",
        "Test data:  Cache-friendly loop ordering",
        "Train data: Blocked Matrix Multiplication",
        "Test data:  Blocked Matrix Multiplication",
    ])

    # Return figure
    plt.savefig(AUTO_DRIVER_DEFAULT_OUTPUT_NAME)
    plt.close()



rafcomputingapi_mock_submit = {
    "program":
    "Bubble sort",
    "name":
    "results_20200415112451",
    "data": [
        {
            "input": 65536,
            "output": 17.035170
        },
        {
            "input": 1048576,
            "output": 4383.926270
        },
    ],
    "sys": {
        "Architecture": "x86_64",
        "CPU op-mode(s)": "32-bit, 64-bit",
        "Byte Order": "Little Endian",
        "CPU(s)": "4",
        "On-line CPU(s) list": "0-3",
        "Thread(s) per core": "1",
        "Core(s) per socket": "4",
        "Socket(s)": "1",
        "NUMA node(s)": "1",
        "Vendor ID": "GenuineIntel",
        "CPU family": "6",
        "Model": "58",
        "Stepping": "9",
        "CPU MHz": "3201.000",
        "BogoMIPS": "6385.77",
        "Virtualization": "VT-x",
        "L1d cache": "32K",
        "L1i cache": "32K",
        "L2 cache": "256K",
        "L3 cache": "6144K",
        "NUMA node0 CPU(s)": "0-3",
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import json
import os

import urlparse
from browsermobproxy import Server
from selenium import webdriver
from xvfbwrapper import Xvfb


class performance(object):
    # create performance data

    def __init__(self, mob_path):
        # initialize
        from datetime import datetime

        print("%s: Go " % (datetime.now()))
        self.browser_mob = mob_path
        self.server = self.driver = self.proxy = None

    @staticmethod
    def __store_into_file(args, title, result):
        # store data collected into file
        if "path" in args:
            har_file = open(args["path"] + "/" + title + ".json", "w")
        else:
            har_file = open(title + ".json", "w")
        har_file.write(result.encode("utf-8"))
        har_file.close()

    def __start_server(self):
        # prepare and start server
        self.server = Server(self.browser_mob)
        self.server.start()
        self.proxy = self.server.create_proxy()

    def __start_driver(self, args):
        # prepare and start driver

        # chromedriver
        if args["browser"] == "chrome":
            print("Browser: Chrome")
            print("URL: {0}".format(args["url"]))
            chromedriver = os.getenv("CHROMEDRIVER_PATH", "/chromedriver")
            os.environ["webdriver.chrome.driver"] = chromedriver
            url = urlparse.urlparse(self.proxy.proxy).path
            chrome_options = webdriver.ChromeOptions()
            chrome_options.add_argument("--proxy-server={0}".format(url))
            chrome_options.add_argument("--no-sandbox")
            self.driver = webdriver.Chrome(chromedriver,
                                           chrome_options=chrome_options)
        # firefox
        if args["browser"] == "firefox":
            print("Browser: Firefox")
            profile = webdriver.FirefoxProfile()
            profile.set_proxy(self.proxy.selenium_proxy())
            self.driver = webdriver.Firefox(firefox_profile=profile)

    def start_all(self, args):
        # start server and driver
        self.__start_server()
        self.__start_driver(args)

    def create_har(self, args):
        # start request and parse response
        self.proxy.new_har(args["url"], options={"captureHeaders": True})
        self.driver.get(args["url"])

        result = json.dumps(self.proxy.har, ensure_ascii=False)
        self.__store_into_file(args, "har", result)

        performance = json.dumps(
            self.driver.execute_script("return window.performance"),
            ensure_ascii=False)
        self.__store_into_file(args, "perf", performance)

    def stop_all(self):
        # stop server and driver
        from datetime import datetime

        print("%s: Finish" % (datetime.now()))

        self.server.stop()
        self.driver.quit()


if __name__ == "__main__":
    # for headless execution
    with Xvfb() as xvfb:
        parser = argparse.ArgumentParser(
            description="Performance Testing using Browsermob-Proxy and Python"
        )
        parser.add_argument("-u", "--url", help="URL to test", required=True)
        parser.add_argument("-b",
                            "--browser",
                            help="Select Chrome or Firefox",
                            required=True)
        parser.add_argument("-p",
                            "--path",
                            help="Select path for output files",
                            required=False)
        args = vars(parser.parse_args())
        path = os.getenv("BROWSERMOB_PROXY_PATH",
                         "/browsermob-proxy-2.1.2/bin/browsermob-proxy")
        RUN = performance(path)
        RUN.start_all(args)
        RUN.create_har(args)
        RUN.stop_all()



import gym

env = gym.make('ChessVsRandomBot-v0')


def make_move():
    state = env.state
    player = 1
    actions = env.get_possible_actions(state, player)
    print(actions)
    for a in actions:
        print(env.action_to_move(a, player))

    for a in actions:
        state, reward, done, __ = env.step(a)
        __ = env.reset()


if __name__ == "__main__":
    make_move()



import matplotlib.pyplot as plt
import sys
from numpy.polynomial import Polynomial as P
from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error, r2_score

from rafComputing.ML.CustomSettings.settings import AUTO_DRIVER_DEFAULT_OUTPUT_NAME
from rafComputing.ML.features.feature_types import (
    NO_FEATURE_TYPE, )
from rafComputing.ML.helpers.load_data import matrix_to_train_test
from rafComputing.ML.helpers.polynomial_to_latex import polynomial_to_LaTeX


def LinearRegressionTrainingAuto(
    path,
    output_name=AUTO_DRIVER_DEFAULT_OUTPUT_NAME,
    feature_type=NO_FEATURE_TYPE,
    feature_val=1.0,
):
    (x, orig_x), y, (X_train, orig_x_train), (
        X_test,
        orig_x_test,
    ), y_train, y_test = matrix_to_train_test(path=path,
                                              feature_type=feature_type,
                                              feature_val=feature_val)

    # Model initialization
    regression_model = LinearRegression(fit_intercept=False)
    regression_model.fit(X_train, y_train)

    # Predict
    y_predicted = regression_model.predict(x)
    y_predicted_train = regression_model.predict(X_train)
    y_predicted_test = regression_model.predict(X_test)

    # Model evaluation training data
    rmse = mean_squared_error(y_train, y_predicted_train)
    r2 = r2_score(y_train, y_predicted_train)
    print("[Training Set] Root mean squared error: ", rmse)
    print("[Training Set] R2 score: ", r2)

    # Model evaluation test data
    rmse = mean_squared_error(y_test, y_predicted_test)
    r2 = r2_score(y_test, y_predicted_test)
    print("[Test Set] Root mean squared error: ", rmse)
    print("[Test Set] R2 score: ", r2)

    orig_y_train = y_train.flatten()
    orig_y_test = y_test.flatten()

    # Data points
    plt.scatter(orig_x_train, orig_y_train, s=20, color="b")
    plt.scatter(orig_x_test, orig_y_test, s=40, color="r")
    plt.xlabel("Input size")
    plt.ylabel("Time (seconds)")

    # Plotting predicted values
    orig_x = orig_x.flatten()

    # Predicted values
    y_predicted = y_predicted.flatten()
    plt.plot(orig_x, y_predicted, color="g")
    plt.legend(["Regression line", "Train data", "Test data"])
    plt.title(polynomial_to_LaTeX(P(regression_model.coef_.flatten())))

    # Return figure
    plt.savefig(output_name)

    plt.close()


if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print("Usage: python3 MLAutoDriver.py <file_name> ")
        sys.exit(-1)

    path = sys.argv[1]
    OUTPUT_NAME = "result.png"

    LinearRegressionTrainingAuto(path,
                                 output_name=OUTPUT_NAME,
                                 feature_type=NO_FEATURE_TYPE,
                                 feature_val=1.0)



import matplotlib.pyplot as plt
from sklearn.metrics import mean_squared_error, r2_score

from rafComputing.ML.CustomSettings.settings import (
    MAX_ITER_LinearRegressionTraining,
    ALPHA_LinearRegressionTraining,
    AUTO_DRIVER_DEFAULT_OUTPUT_NAME,
)
from rafComputing.ML.RegressionEngine.LinearRegressionGD import LinearRegressionGD
from rafComputing.ML.features.feature_types import (
    NO_FEATURE_TYPE, )
from rafComputing.ML.helpers.generate_larger_evaluation_set import generate_larger_evaluation_set
from rafComputing.ML.helpers.load_data import matrix_to_train_test_w_generate_larger_evaluation_set
from rafComputing.ML.snippets.MLdriver_more_graphs_prediction import TEST_POINT_COLORS, LINE_COLORS


def CustomStrassenLinearRegressionTrainingMoreGraphsPrediction(
    path,
    alpha=ALPHA_LinearRegressionTraining,
    n_iterations=MAX_ITER_LinearRegressionTraining,
    internal_counter=0,
    feature_type=NO_FEATURE_TYPE,
    feature_val=1.0,
    _first=True,
    final=False,
    plt_legend=None,
    custom_generate_larger_evaluation_set=generate_larger_evaluation_set,
):
    (x, orig_x), y, (X_train, orig_x_train), (
        X_test,
        orig_x_test,
    ), y_train, y_test, (
        X_test2,
        orig_x_test2) = matrix_to_train_test_w_generate_larger_evaluation_set(
            path=path,
            feature_type=feature_type,
            feature_val=feature_val,
            custom_generate_larger_evaluation_set=
            custom_generate_larger_evaluation_set)

    # Model initialization
    regression_model = LinearRegressionGD(alpha, n_iterations)
    regression_model.fit(X_train, y_train)
    print(regression_model.w_.flatten())

    y_predicted = regression_model.predict(x)
    y_predicted_train = regression_model.predict(X_train)
    y_predicted_test = regression_model.predict(X_test)

    # Predict generated sets
    y_predicted_test2 = regression_model.predict(X_test2)

    # Model evaluation training data
    rmse = mean_squared_error(y_train, y_predicted_train)
    r2 = r2_score(y_train, y_predicted_train)
    print("[Training Set] Root mean squared error: ", rmse)
    print("[Training Set] R2 score: ", r2)

    # Model evaluation test data
    rmse = mean_squared_error(y_test, y_predicted_test)
    r2 = r2_score(y_test, y_predicted_test)
    print("[Test Set] Root mean squared error: ", rmse)
    print("[Test Set] R2 score: ", r2)

    orig_y_train = y_train.flatten()
    orig_y_test = y_test.flatten()

    # Data points
    plt.scatter(orig_x_test2,
                y_predicted_test2,
                marker='x',
                s=16,
                color=TEST_POINT_COLORS[internal_counter])

    plt.xlabel("Input size")
    plt.ylabel("Time (seconds)")

    # Plotting predicted values
    orig_x = orig_x.flatten()

    # Predicted values
    plt.plot(orig_x_test2,
             y_predicted_test2,
             color=LINE_COLORS[internal_counter],
             linestyle="--")

    if final:
        plt.legend(plt_legend, fontsize=8)

        # Return figure
        plt.savefig(AUTO_DRIVER_DEFAULT_OUTPUT_NAME)
        plt.close()



from mysql_config.WebMonitoring.generators.resource.resource_generate_samples_queries import (
    resource_generate_samples_queries, )
from mysql_config.WebMonitoring.generators.resource.resource_generate_samples_queries_size import (
    resource_generate_samples_queries_size, )
from mysql_config.WebMonitoring.generators.resource.tests.resource_expected_size import (
    EXPECTED_DAILY_RESOURCE_GENERATE_SAMPLES_QUERIES_SIZE,
    EXPECTED_WEEKLY_RESOURCE_GENERATE_SAMPLES_QUERIES_SIZE,
    EXPECTED_MONTHLY_RESOURCE_GENERATE_SAMPLES_QUERIES_SIZE,
)
from mysql_config.WebMonitoring.generators.resource.tests.resource_expected_time import (
    EXPECTED_DAILY_RESOURCE_GENERATE_SAMPLES_QUERIES,
    EXPECTED_WEEKLY_RESOURCE_GENERATE_SAMPLES_QUERIES,
    EXPECTED_MONTHLY_RESOURCE_GENERATE_SAMPLES_QUERIES,
)


def test_resource_generate_samples_queries(capfd):
    resource_generate_samples_queries("daily")
    out, _ = capfd.readouterr()
    assert out == EXPECTED_DAILY_RESOURCE_GENERATE_SAMPLES_QUERIES

    resource_generate_samples_queries("weekly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_WEEKLY_RESOURCE_GENERATE_SAMPLES_QUERIES

    resource_generate_samples_queries("monthly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_MONTHLY_RESOURCE_GENERATE_SAMPLES_QUERIES


def test_resource_generate_samples_queries_size(capfd):
    resource_generate_samples_queries_size("daily")
    out, _ = capfd.readouterr()
    assert out == EXPECTED_DAILY_RESOURCE_GENERATE_SAMPLES_QUERIES_SIZE

    resource_generate_samples_queries_size("weekly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_WEEKLY_RESOURCE_GENERATE_SAMPLES_QUERIES_SIZE

    resource_generate_samples_queries_size("monthly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_MONTHLY_RESOURCE_GENERATE_SAMPLES_QUERIES_SIZE



import matplotlib.pyplot as plt
import numpy as np
import sys

from rafComputing.ML.CustomSettings.settings import (
    MAX_ITER,
    DEFAULT_ITER_INCREASE_STEPS_LOG,
    DEFAULT_ITER_INCREASE_STEPS,
    DEFAULT_RANGE_,
    MAX_ITER_LinearRegressionTraining,
    ALPHA_LinearRegressionTraining,
    OUTPUT_PREFIX,
    OUTPUT_PREFIX_0,
)
from rafComputing.ML.RegressionEngine.LinearRegressionGD import LinearRegressionGD
from rafComputing.ML.features.feature_transformation import extract_features
from rafComputing.ML.features.feature_types import (NO_FEATURE_TYPE,
                                                    POWER_FEATURE_TYPE)
from rafComputing.ML.helpers.load_data import matrix_to_train_test


def LinearRegressionTraining(
    path,
    alpha=ALPHA_LinearRegressionTraining,
    n_iterations=MAX_ITER_LinearRegressionTraining,
    output_name=OUTPUT_PREFIX_0,
    feature_type=NO_FEATURE_TYPE,
    feature_val=1.0,
):
    (x, orig_x), y, (X_train, orig_x_train), (
        X_test,
        orig_x_test,
    ), y_train, y_test = matrix_to_train_test(path=path,
                                              feature_type=feature_type,
                                              feature_val=feature_val)

    # Model initialization
    regression_model = LinearRegressionGD(alpha, n_iterations)
    regression_model.fit(X_train, y_train)

    print(regression_model.w_)

    # Add additional generated data
    support_x = np.linspace(0, np.max(orig_x), 100)
    features_support_x = extract_features(support_x, feature_type, feature_val)
    support_y_predicted = regression_model.predict(features_support_x)

    # Predict
    y_predicted = regression_model.predict(x)
    # y_predicted_train = regression_model.predict(X_train)
    # y_predicted_test = regression_model.predict(X_test)

    orig_y_train = y_train.flatten()
    orig_y_test = y_test.flatten()

    # Data points

    plt.scatter(np.concatenate([orig_x_train, orig_x_test]),
                np.concatenate([orig_y_train, orig_y_test]),
                s=20,
                color="b")
    """

    plt.scatter(np.concatenate([orig_x_train, orig_x_test]),
                np.concatenate([y_predicted_train, y_predicted_test]),
                s=20,
                color="r")
    """
    plt.xlabel("Input size")
    plt.ylabel("Memory (Gb)")

    # Plotting predicted values
    orig_x = orig_x.flatten()

    # Predicted values
    y_predicted = y_predicted.flatten()
    plt.plot(support_x, support_y_predicted, color="g")
    plt.legend(["Regression line", "Predictied datapoint"])

    plt.title(
        "Memory usage: Strassen algorithm. \nComplexity function = n^log2(7)")

    # Return figure
    plt.savefig(output_name)

    plt.close()


if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print("Usage v1: python3 MLdriver_strassen_memory.py <file_name> ")
        print(
            "[DEPRECATED FOR MLdriver_strassen_memory] Usage v2: python3 MLdriver_strassen_memory.py <file_name> <ITER_INCREASE_STEPS_LOG = 0 default>"
        )
        print(
            "Usage frequent: python3 rafComputing/ML/snippets/demo/strassen/MLdriver_strassen_memory.py rComplexity/samples/matrix_multiplication/results/sprmcrogpu-wn13/result_mem_n28_GB_20200417113226 "
        )

        sys.exit(-1)
    path = sys.argv[1]

    ITER_INCREASE_STEPS_LOG = DEFAULT_ITER_INCREASE_STEPS_LOG
    ITER_INCREASE_STEPS = DEFAULT_ITER_INCREASE_STEPS
    range_ = DEFAULT_RANGE_

    for counter in range_:
        output_name = OUTPUT_PREFIX + str(counter).zfill(
            ITER_INCREASE_STEPS_LOG)
        LinearRegressionTraining(
            path=path,
            alpha=1e-18,
            n_iterations=int(MAX_ITER / ITER_INCREASE_STEPS * counter + 1),
            output_name=output_name,
            feature_type=POWER_FEATURE_TYPE,
            feature_val=2.8,
        )



import os

# DB configs
MYSQL_DATABASE_USER = "root"
MYSQL_DATABASE_PASSWORD = "password"
MYSQL_DATABASE_DB = "WebMonitoring"
if os.environ.get("DOCKER_COMPOSE_BUILD"):
    MYSQL_DATABASE_HOST = "mysql"
else:
    MYSQL_DATABASE_HOST = "10.96.0.2"



from WebMonitoring.API.constants import PERIOD
from WebMonitoring.API.resources.helpers import get_timestamp_query


def websites_get_samples_time_daily(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [
            resource_name,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ]

        cursor.callproc("get_daily_samples_websites", args)
        cursor.execute("SELECT "
                       "@_get_daily_samples_websites_0, "
                       "@_get_daily_samples_websites_1, "
                       "@_get_daily_samples_websites_2, "
                       "@_get_daily_samples_websites_3, "
                       "@_get_daily_samples_websites_4, "
                       "@_get_daily_samples_websites_5, "
                       "@_get_daily_samples_websites_6, "
                       "@_get_daily_samples_websites_7, "
                       "@_get_daily_samples_websites_8, "
                       "@_get_daily_samples_websites_9, "
                       "@_get_daily_samples_websites_10, "
                       "@_get_daily_samples_websites_11, "
                       "@_get_daily_samples_websites_12, "
                       "@_get_daily_samples_websites_13, "
                       "@_get_daily_samples_websites_14, "
                       "@_get_daily_samples_websites_15, "
                       "@_get_daily_samples_websites_16, "
                       "@_get_daily_samples_websites_17, "
                       "@_get_daily_samples_websites_18, "
                       "@_get_daily_samples_websites_19, "
                       "@_get_daily_samples_websites_20, "
                       "@_get_daily_samples_websites_21, "
                       "@_get_daily_samples_websites_22, "
                       "@_get_daily_samples_websites_23, "
                       "@_get_daily_samples_websites_24, "
                       "@_get_daily_samples_websites_25 ")
        result_args_get_time = cursor.fetchone()

        result = []
        for i in range(len(result_args_get_time) - 2):
            result.append({
                "custom_data":
                result_args_get_time[i + 1],
                "label":
                get_timestamp_query(PERIOD.DAILY, i, result_args_get_time[-1]),
            })

        return result
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None


def websites_get_samples_time_weekly(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [
            resource_name,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ]

        cursor.callproc("get_weekly_samples_websites", args)
        cursor.execute("SELECT "
                       "@_get_weekly_samples_websites_0, "
                       "@_get_weekly_samples_websites_1, "
                       "@_get_weekly_samples_websites_2, "
                       "@_get_weekly_samples_websites_3, "
                       "@_get_weekly_samples_websites_4, "
                       "@_get_weekly_samples_websites_5, "
                       "@_get_weekly_samples_websites_6, "
                       "@_get_weekly_samples_websites_7, "
                       "@_get_weekly_samples_websites_8, "
                       "@_get_weekly_samples_websites_9, "
                       "@_get_weekly_samples_websites_10, "
                       "@_get_weekly_samples_websites_11, "
                       "@_get_weekly_samples_websites_12, "
                       "@_get_weekly_samples_websites_13, "
                       "@_get_weekly_samples_websites_14, "
                       "@_get_weekly_samples_websites_15, "
                       "@_get_weekly_samples_websites_16, "
                       "@_get_weekly_samples_websites_17, "
                       "@_get_weekly_samples_websites_18, "
                       "@_get_weekly_samples_websites_19, "
                       "@_get_weekly_samples_websites_20, "
                       "@_get_weekly_samples_websites_21, "
                       "@_get_weekly_samples_websites_22, "
                       "@_get_weekly_samples_websites_23, "
                       "@_get_weekly_samples_websites_24, "
                       "@_get_weekly_samples_websites_25, "
                       "@_get_weekly_samples_websites_26, "
                       "@_get_weekly_samples_websites_27, "
                       "@_get_weekly_samples_websites_28, "
                       "@_get_weekly_samples_websites_29 ")
        result_args_get_time = cursor.fetchone()

        result = []
        for i in range(len(result_args_get_time) - 2):
            result.append({
                "custom_data":
                result_args_get_time[i + 1],
                "label":
                get_timestamp_query(PERIOD.WEEKLY, i,
                                    result_args_get_time[-1]),
            })
        return result
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None


def websites_get_samples_time_monthly(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [
            resource_name,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ]

        cursor.callproc("get_monthly_samples_websites", args)
        cursor.execute("SELECT "
                       "@_get_monthly_samples_websites_0, "
                       "@_get_monthly_samples_websites_1, "
                       "@_get_monthly_samples_websites_2, "
                       "@_get_monthly_samples_websites_3, "
                       "@_get_monthly_samples_websites_4, "
                       "@_get_monthly_samples_websites_5, "
                       "@_get_monthly_samples_websites_6, "
                       "@_get_monthly_samples_websites_7, "
                       "@_get_monthly_samples_websites_8, "
                       "@_get_monthly_samples_websites_9, "
                       "@_get_monthly_samples_websites_10, "
                       "@_get_monthly_samples_websites_11, "
                       "@_get_monthly_samples_websites_12, "
                       "@_get_monthly_samples_websites_13, "
                       "@_get_monthly_samples_websites_14, "
                       "@_get_monthly_samples_websites_15, "
                       "@_get_monthly_samples_websites_16, "
                       "@_get_monthly_samples_websites_17, "
                       "@_get_monthly_samples_websites_18, "
                       "@_get_monthly_samples_websites_19, "
                       "@_get_monthly_samples_websites_20, "
                       "@_get_monthly_samples_websites_21, "
                       "@_get_monthly_samples_websites_22, "
                       "@_get_monthly_samples_websites_23, "
                       "@_get_monthly_samples_websites_24, "
                       "@_get_monthly_samples_websites_25, "
                       "@_get_monthly_samples_websites_26, "
                       "@_get_monthly_samples_websites_27, "
                       "@_get_monthly_samples_websites_28, "
                       "@_get_monthly_samples_websites_29, "
                       "@_get_monthly_samples_websites_30, "
                       "@_get_monthly_samples_websites_31, "
                       "@_get_monthly_samples_websites_32")
        result_args_get_time = cursor.fetchone()

        result = []
        for i in range(len(result_args_get_time) - 2):
            result.append({
                "custom_data":
                result_args_get_time[i + 1],
                "label":
                get_timestamp_query(PERIOD.MONTHLY, i,
                                    result_args_get_time[-1]),
            })
        return result
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None



from mysql_config.WebMonitoring.generators.website.tests.website_expected_size import (
    EXPECTED_DAILY_WEBSITE_GENERATE_SAMPLES_QUERIES_SIZE,
    EXPECTED_WEEKLY_WEBSITE_GENERATE_SAMPLES_QUERIES_SIZE,
    EXPECTED_MONTHLY_WEBSITE_GENERATE_SAMPLES_QUERIES_SIZE,
)
from mysql_config.WebMonitoring.generators.website.tests.website_expected_time import (
    EXPECTED_DAILY_WEBSITE_GENERATE_SAMPLES_QUERIES,
    EXPECTED_WEEKLY_WEBSITE_GENERATE_SAMPLES_QUERIES,
    EXPECTED_MONTHLY_WEBSITE_GENERATE_SAMPLES_QUERIES,
)
from mysql_config.WebMonitoring.generators.website.website_generate_samples_queries import (
    website_generate_samples_queries, )
from mysql_config.WebMonitoring.generators.website.website_generate_samples_queries_size import (
    website_generate_samples_queries_size, )


def test_website_generate_samples_queries(capfd):
    website_generate_samples_queries("daily")
    out, _ = capfd.readouterr()
    assert out == EXPECTED_DAILY_WEBSITE_GENERATE_SAMPLES_QUERIES

    website_generate_samples_queries("weekly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_WEEKLY_WEBSITE_GENERATE_SAMPLES_QUERIES

    website_generate_samples_queries("monthly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_MONTHLY_WEBSITE_GENERATE_SAMPLES_QUERIES


def test_website_generate_samples_queries_size(capfd):
    website_generate_samples_queries_size("daily")
    out, _ = capfd.readouterr()
    assert out == EXPECTED_DAILY_WEBSITE_GENERATE_SAMPLES_QUERIES_SIZE

    website_generate_samples_queries_size("weekly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_WEEKLY_WEBSITE_GENERATE_SAMPLES_QUERIES_SIZE

    website_generate_samples_queries_size("monthly")
    out, err = capfd.readouterr()
    assert out == EXPECTED_MONTHLY_WEBSITE_GENERATE_SAMPLES_QUERIES_SIZE
